/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "Parser.h"
#include "Mutex.h"

void Remove_Trailing_Spaces(char *String) {
	int i;
	for (i=strlen(String)-2; i>=0; i--) {
		if (String[i]==' ') String[i]=0;
			else break;
	}
}

int String_Exists(char *String, int *NumberStrings, char ***Stringlist) {
	int i;
	for (i=0; i<(*NumberStrings); i++) {
		if (strcasecmp((*Stringlist)[i], String)==0) return 1;
	}
	return 0;
}

void Add_String(char *String, int *NumberStrings, char ***Stringlist) {
	Lock_Stringlist();
	if (!String_Exists(String, NumberStrings, Stringlist)) {
		(*Stringlist)=(char**)realloc((*Stringlist), sizeof(char**)*((*NumberStrings)+1));
		(*Stringlist)[(*NumberStrings)] = (char*)malloc((sizeof(char)*strlen(String))+1);
		strcpy((*Stringlist)[(*NumberStrings)], String);
		(*NumberStrings)++;
	}
	Unlock_Stringlist();
}

int Parse_Groupinfo(char *Filename, char *Workgroup) {
	FILE *f;
	char Buffer[1024];

	f = fopen(Filename, "r");
	if (f==NULL) {
		fprintf(stderr, "Error parsing master browser file '%s': %s\n", Filename, strerror(errno));
		return 1;
	}

	while (fgets(Buffer, 1023, f) != NULL) {
		if (strlen(Buffer)>=32) {
			if (strncmp(Buffer+17, "<00> - <GROUP>", 14)==0) {
				Buffer[16]=0;
				Remove_Trailing_Spaces(Buffer);
				strncpy(Workgroup, Buffer+1, 255);
				break;
			}
		}
	}

	fclose(f);
	return 0;
}

int Parse_Masterbrowser(char *Filename, int *NumberHosts, char ***Hostlist) {
	FILE *f;
	char Buffer[1024];
	
	f = fopen(Filename, "r");
	if (f==NULL) {
		fprintf(stderr, "Error parsing master browser file '%s': %s\n", Filename, strerror(errno));
		return 1;
	}

	while (fgets(Buffer, 1023, f) != NULL) {
		if (strlen(Buffer)>=32) {
			if (strncmp(Buffer+17, "<00> -        ", 14)==0) {
				Buffer[16]=0;
				Remove_Trailing_Spaces(Buffer);
				Add_String(Buffer+1, NumberHosts, Hostlist);
			}
		}
	}

	fclose(f);
	return 0;
}

int Parse_Hostinfo(char *Hostname, char *Filename, int *NumberHosts, char ***Hostlist, int *NumberShares, char ***Sharelist) {
	FILE *f;
	char Buffer[1024];
	char Buffer2[1024];
	char *String;
	
	f = fopen(Filename, "r");
	if (f==NULL) {
		fprintf(stderr, "Error parsing host info file '%s': %s\n", Filename, strerror(errno));
		return 1;
	}

	while (fgets(Buffer, 1023, f) != NULL) {
		String = strtok(Buffer, "|");
		if (String!=NULL) {
			if (strcmp(String, "Server")==0) {
				String = strtok(NULL, "|");
				if (String!=NULL) {
					Add_String(String, NumberHosts, Hostlist);
				}
			}
			if (strcmp(String, "Disk")==0) {
				String = strtok(NULL, "|");
				if (String!=NULL) {
					snprintf(Buffer2, 1023, "//%s/%s", Hostname, String);
					Add_String(Buffer2, NumberShares, Sharelist);
				}
			}
		}
	}

	fclose(f);
	return 0;
}

